/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * Generic tree selection combo box. This combo uses icons to represent the
 * selected value. The icon class is obtained from the tree node's attribute
 * whose name is configured in the iconClsField configuration option.
 *
 * Inspired by the code published at http://www.sencha.com/forum/showthread.php?38654-Tree-in-a-Combo./page6
 * and based on the IconCombo.
 */
Ext.define( 'Ext.ux.form.TreeIconCombo', {
  extend : 'Ext.form.field.Picker',

  alias : 'widget.TreeIconCombo',

  editable : false,
  forceSelection : true,

  treeWidth : 240,

  /**
   * Style to automatically apply to the combo's input field in case
   * the value is not found in the store:
   * - store load failed
   * - store load successful
   * - store is loading loaded
   * - combo has an empty value
   */
  valueNotInStoreCls : 'value-not-in-store',

  /**
   * Text rendered in the combo's text field in case the value is empty (according to the Ext.isEmpty method).
   */
  emptyValueText : 'Please select',

  /**
   * Text rendered in the combo's text field in case the value is not found in the
   * store.
   */
  valueNotFoundText : 'Value not found',

  /**
   * Text rendered in the combo's text field in case the store load failed.
   */
  loadFailedText : 'Data load failed',

  /**
   * Text rendered in the combo's text field while the store is being loaded.
   */
  loadingText : 'Loading...',

  /**
   * Optional prefix that is used to lookup a tree node by the combo value.
   * The prefix is used to construct the value of the tree node ID by which
   * the tree node is looked up. The lookup node ID is constructed as follows:
   *
   * nodeId = idValuePrefix + <combo value>
   */
  idValuePrefix : null,

  /**
   * Name of the tree node's property whose value is used as the combo value.
   */
  valueField : 'value',

  /**
   * Name of the tree node's property whose value is used as the combo display text.
   */
  displayField : 'name',

  /**
   * Name of the tree node's property whose value is used to create a combo icon.
   */
  iconClsField : 'iconCls',


  storeLoadFinished : false,
  storeLoadSuccessful : false,

  /**
   * Temporary storage of the combo value while the store is loaded.
   */
  futureValue : null,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.root )
      Ext.Error.raise( 'Required config parameter missing: root' );

    if ( !config.store )
      Ext.Error.raise( 'Required config parameter missing: store' );

    Ext.apply( me, config );

    me.callParent( [config] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.getStore().on( 'beforeload', function ( store, operation, eOpts )
    {
      me.storeLoadFinished = false;
      me.storeLoadSuccessful = false;
    } );

    // upon loading the store, determine the selected combo value
    me.getStore().on( 'load', function ( store, root, records, successful, eOpts )
    {
      me.storeLoadSuccessful = successful;
      me.storeLoadFinished = true;

      // if load was successful, set the future value as the combo's value
      if ( successful )
      {
        me.setValue( me.futureValue );
      }
      else
      {
        // load failed, the combo value remains unchanged - we still need to call
        // setValue to update the icon and text
        var value = me.getValue();
        me.setValue( value );

        // Important to force field validation even if the value has not been changed.
        // It is to disable all formBind form elements while the store is loading.
        // While the store is loading, this field's getErrors returns a validation error
        // that makes the formBind fields to be disabled.
        me.validate();
      }
    } );

    // possibly create a picker
    me.getPicker();
  },


  getStore : function ()
  {
    return this.store;
  },


  createPicker : function ()
  {
    var me = this;

    return new Ext.tree.Panel( {
      height : 300,
      autoScroll : true,
      floating : true,
      resizable : true,
      focusOnToFront : false,
      shadow : true,
      ownerCt : me.ownerCt,
      useArrows : false,
      store : me.store,
      root : me.root,
      rootVisible : true,
      listeners : {
        scope : me,
        select : me._valueSelectedInPicker
      },
      displayField : me.displayField
    } );
  },


  getPicker : function ()
  {
    var me = this;
    return me.callParent();
  },


  alignPicker : function ()
  {
    // override the original method because otherwise the height of the treepanel would be always 0
    var me = this, picker, isAbove, aboveSfx = '-above';
    if ( me.isExpanded )
    {
      picker = me.getPicker();
      if ( me.matchFieldWidth )
      {
        // Auto the height (it will be constrained by min and max width) unless there are no records to display.
        if ( me.bodyEl.getWidth() > me.treeWidth )
        {
          picker.setWidth( me.bodyEl.getWidth() );
        }
        else picker.setWidth( me.treeWidth );
      }
      if ( picker.isFloating() )
      {
        picker.alignTo( me.inputEl, "", me.pickerOffset );// ""->tl
        // add the {openCls}-above class if the picker was aligned above the field due to hitting the bottom of the viewport
        isAbove = picker.getEl().getY() < me.inputEl.getY();
        me.bodyEl[isAbove ? 'addCls' : 'removeCls']( me.openCls
        + aboveSfx );
        picker.getEl()[isAbove ? 'addCls' : 'removeCls']( picker.baseCls
        + aboveSfx );
      }
    }
  },


  onRender : function ( ct, position )
  {
    var me = this;

    // call parent onRender
    me.callParent( arguments );

    // adjust styles
    //this.bodyEl.applyStyles( {position : 'relative'} );
    me.getEl().down( 'input.x-form-field' ).addCls( 'ux-tree-icon-combo-input' );
    me.getEl().down( '.x-form-trigger-wrap' ).applyStyles( {position : 'relative'} );

    // add span for icon
    me.icon = Ext.core.DomHelper.insertBefore( me.getEl().down( '.x-form-text-wrap input' ), {
      tag : 'span', style : 'position:absolute'
    } );
  },


  /**
   * Invoked by ExtJS after the picker has been expanded and shown. This method selects and expands
   * the tree node with the current combo's value.
   */
  onExpand : function ()
  {
    var me = this;

    var value = me.getValue();
    var valueNodeId = me._createNodeIdForValue( value );
    var valueNode = me.getStore().getNodeById( valueNodeId );

    if ( valueNode )
    {
      var picker = me.getPicker();
      var selectionModel = picker.getSelectionModel();

      selectionModel.deselectAll( true );  // suppress event (would collapse the picker if opened)
      selectionModel.select( valueNode, false, true ); // suppress event (would collapse the picker if opened)

      picker.expandPath( valueNode.getPath() );
    }
  },


  /**
   * Invoked by ExtJS after the picker has been collapsed and hidden. This method collapses and
   * nodes so that when the picker is expanded next time, only the value node path is expanded.
   */
  onCollapse : function ()
  {
    var me = this;
    var picker = me.getPicker();
    picker.collapseAll();
  },


  getValue : function ()
  {
    var me = this;
    return me.value;
  },


  getSubmitValue : function ()
  {
    var me = this;
    var value = me.getValue();
    return value ? value.toString() : null;
  },


  setValue : function ( value )
  {
    var me = this;

    if ( me.storeLoadFinished )
    {
      me.value = me.futureValue = value;
      me.callParent( arguments );
    }
    else
    {
      // store load in progress - save the value in a temp storage and set the combo's raw value;
      // once the store is loaded the saved value will be set (by invoking this method)
      me.futureValue = value;

      me.setRawValue( me.valueToRaw( value ) );
    }

    me._syncIconCls();
  },


  /**
   * Invoked by setValue to obtain the raw value for the input field.
   * @param value a value.
   * @return the raw value.
   */
  valueToRaw : function ( value )
  {
    var me = this;

    if ( !Ext.isEmpty( value ) )
    {
      var record = me._findRecordByValue( value );
      if ( record )
      {
        return record.get( me.displayField );
      }
    }

    // Value is not empty and it is not found in the store. The reasons can be:
    // 1. Store is not loaded yet.
    // 2. Store load failed.
    // 3. Store load succeeded, but the value is not present.

    if ( me.storeLoadFinished )
    {
      if ( me.storeLoadSuccessful )
      {
        if ( Ext.isEmpty( value ) )
        {
          return me.emptyValueText;
        }

        // store load successful, but the value is not found in the store
        return me.valueNotFoundText;
      }
      else
      {
        // store load failed and the value is not found in the store
        return me.loadFailedText;
      }
    }
    else
    {
      // store load in progress
      return me.loadingText;
    }
  },


  /**
   * Invoked by the ExtJS validation framework to validate this field.
   * @param processedRawValue the raw field value.
   * @return an array with error messages.
   */
  getErrors : function ( processedRawValue )
  {
    var me = this;
    var errors = me.callParent( arguments );

    var value = me.getValue();

    if ( me.storeLoadFinished )
    {
      if ( me.storeLoadSuccessful )
      {
        if ( Ext.isEmpty( value ) )
        {
          if ( !me.allowBlank )
          {
            errors.push( me.emptyValueText );
          }
        }
        else
        {
          // store load successful and value not empty - check the value is in the store
          var record = me._findRecordByValue( value );
          if ( !record )
          {
            errors.push( me.valueNotFoundText );
          }

          // additional value validations should be added here...
        }
      }
      else
      {
        // store load failed
        errors.push( me.loadFailedText );
      }
    }
    else
    {
      // store load in progress - validation error prevents form submit
      errors.push( me.loadingText );
    }

    return errors;
  },


  /**
   * Returns the currently selected tree node (record) for the current combo value.
   * @return {*} the tree node (record) for the current combo value.
   */
  getSelectedRecord : function ()
  {
    var me = this;
    var value = me.getValue();
    return me._findRecordByValue( value );
  },


  _syncIconCls : function ()
  {
    var me = this;
    if ( me.rendered )
    {
      var value = me.getValue();

      if ( !Ext.isEmpty( value ) )
      {
        var record = me._findRecordByValue( value );
        if ( record )
        {
          me.icon.className = 'ux-tree-icon-combo-icon ' + record.get( me.iconClsField );
          me.inputEl.removeCls( me.valueNotInStoreCls );
          return;
        }
      }

      me.inputEl.addCls( me.valueNotInStoreCls );

      // Value not found in the store. The reasons can be:
      // 1. Store is not loaded yet.
      // 2. Store load failed.
      // 3. Store load succeeded, but the value is not present.
      var iconClassName;
      if ( me.storeLoadFinished )
      {
        if ( me.storeLoadSuccessful )
        {
          if ( Ext.isEmpty( value ) )
          {
            iconClassName = 'empty-value';
          }
          else
          {
            // store load successful, but the value is not found in the store
            iconClassName = 'value-not-found';
          }
        }
        else
        {
          // store load failed and the value is not found in the store
          iconClassName = 'load-failed';
        }
      }
      else
      {
        // store load in progress
        iconClassName = 'loading';
      }

      me.icon.className = 'ux-tree-icon-combo-icon ' + iconClassName;
    }
    else
    {
      me.on( 'render', me._syncIconCls, me, {single : true} );
    }
  },


  /**
   * Attempts to find a tree node (record) for the specified combo value.
   * @param value a combo value.
   * @return the tree node with the specified value, or null if not found.
   * @private
   */
  _findRecordByValue : function ( value )
  {
    var me = this;

    if ( !me.storeLoadFinished || me.getStore().isLoading() )
    {
      return null;
    }
    else
    {
      var valueNodeId = me._createNodeIdForValue( value );
      return me.store.getNodeById( valueNodeId );
    }
  },


  /**
   * Creates a tree node ID for the specified combo value.
   * @param value a combo value.
   * @return {string} the created node ID.
   * @private
   */
  _createNodeIdForValue : function ( value )
  {
    var me = this;
    var nodeId = Ext.isEmpty( value ) ? '' : value;

    if ( !Ext.isEmpty( me.idValuePrefix ) )
    {
      nodeId = me.idValuePrefix + nodeId.toString();
    }

    return nodeId;
  },


  /**
   * Invoked when a record is selected in the picker.
   * @param picker a picker.
   * @param record the selected record.
   * @param options options.
   * @private
   */
  _valueSelectedInPicker : function ( picker, record, options )
  {
    var me = this;
    var value = record.get( me.valueField );
    if ( value )
    {
      me.setValue( value );
      me.fireEvent( 'valueSelected', me, value );
      Ext.Function.defer( function ()
      {
        me.collapse();
      }, 200, me );
    }
  }
} )
;