/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Saves the specified chart in the format described by the specified MIME type.
 * Currently the following MIME types are supported:
 *
 * image/jpeg
 * image/png
 * image/svg+xml
 */
Ext.define( 'QDesk.controller.command.SaveChart', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /**
   * Required config property containing the chart to save.
   */
  chart : undefined,

  /**
   * Required config property containing the MIME type
   * used to save the chart. If the MIME type is a raster image MIME type, then
   * the save operation rasterizes the chart. If the MIME type is SVG, then
   * the chart is saved in SVG format.
   */
  mimeType : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.chart )
      Ext.Error.raise( 'Required config parameter missing: chart' );

    if ( !config.mimeType )
      Ext.Error.raise( 'Required config parameter missing: mimeType' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;
    me._saveChart();
  },


  _saveChart : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.SaveChart.maskMsg' ) );

    /*
     * This no longer works in ExtJS 5. We could instead use:
     *
     * me.chart.getImage('stream' ).data - produces data with the following format "data:application/octet-stream;base64,iV...==="
     * me.chart.getImage().data - produces data with the following format: "data:image/png;base64,iVBORw....==="
     */
    var chartSvgData = Ext.draw.engine.SvgExporter.generate( me.chart.surface, {/* currently no options supported */} );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.svgActionPrepare,
      params : {
        mimeType : me.mimeType,
        width : me.chart.getWidth(),
        height : me.chart.getHeight(),
        svg : chartSvgData
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var handle = json.data.handle;
            me._downloadChart( handle );

            QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.SaveChart.successMsg' ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.SaveChart.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  _downloadChart : function ( handle )
  {
    var downloadUrl = QDesk.util.constant.ActionUrl.svgActionDownload + '&handle=' + handle;

    var iframe = Ext.getBody().createChild( {
      tag : 'iframe',
      width : 0,
      height : 0,
      frameborder : 0,
      src : downloadUrl,
      cls : Ext.baseCSSPrefix + 'hide-display'
    } );

    // give the backend 10 seconds to initiate the download and then silently drop the iframe
    Ext.defer( function ()
        {
          iframe.remove();
        },
        10000 );
  }
} );
