/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Adds new, or updates an existing scheduler connection.
 */
Ext.define( 'QDesk.controller.command.connection.SaveSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.connection.RefreshConnectionFolder',
    'QDesk.util.UxUtils',
    'QDesk.enums.ServerErrorType'
  ],

  /**
   * Required config parameter when saving new scheduler connection.
   */
  parentFolderId : undefined,

  /**
   * Required config parameter with the connection to add/update.
   */
  connection : undefined,

  /**
   * Required config parameter with the EditSchedulerConnectionWindow.
   */
  window : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connection )
      Ext.Error.raise( 'Required config parameter missing: connection' );

    if ( !config.window )
      Ext.Error.raise( 'Required config parameter missing: window' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    if ( Ext.isNumber( me.connection.get( 'id' ) ))
    {
      me._updateConnection();
    }
    else
    {
      me._addConnection();
    }
  },


  _addConnection : function ()
  {
    var me = this;

    var connectionName = me.connection.get( 'name' );

    /*
     * Important to drop id property from the data to avoid JSOn deserialization issues on the server-side.
     */
    var connectionData = me.connection.getData();
    delete connectionData.id;

    QDesk.util.UxUtils.showWaitDialog( QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.addWaitMsg',
        connectionName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionAddConnection,
      params : {
        parentFolderId : me.parentFolderId,
        connection : Ext.encode( connectionData )
      },
      callback : function ( opts, success, resp )
      {
        // hide the wait dialog
        Ext.MessageBox.hide();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me.window.close();

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.addSuccessMsg',
                    connectionName ),
                function ()
                {
                  me.application.fireEvent( 'connectionAdded', {
                    connectionId : json.data.id
                  } );
                } );

            /*
             * Refresh connection's parent folder in the connections tree view.
             */
            Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
              folderId : me.parentFolderId
            } ).execute();
          }
          else
          {
            if ( json.errorType && json.errorType == QDesk.enums.ServerErrorType.VALIDATION )
            {
              Ext.each( Ext.ComponentQuery.query( 'form', me.window ), function ( formPanel )
              {
                formPanel.getForm().markInvalid( json.errors );
              } );
            }

            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.addErrorMsg',
                  connectionName ) );
        }
      }
    } );
  },


  _updateConnection : function ()
  {
    var me = this;

    var connectionId = me.connection.get( 'id' );
    var connectionName = me.connection.get( 'name' );

    QDesk.util.UxUtils.showWaitDialog( QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.updateWaitMsg',
        connectionName ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionUpdateConnection,
      params : {
        connectionId : connectionId,
        connection : Ext.encode( me.connection.getData() )
      },
      callback : function ( opts, success, resp )
      {
        // hide the wait dialog
        Ext.MessageBox.hide();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me.window.close();

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.updateSuccessMsg',
                    connectionName ),
                function ()
                {
                  me.application.fireEvent( 'connectionUpdated', {
                    connectionId : connectionId
                  } );
                } );

            /*
             * Close the scheduler view if it is open.
             */
            me._closeSchedulerTabForConnection( connectionId );

            /*
             * Refresh connection's parent folder in the connections tree view.
             */
            var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getConnectionParentFolderId( connectionId );
            if ( parentFolderId )
            {
              Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
                folderId : parentFolderId
              } ).execute();
            }
            else
            {
              Ext.Error.raise( 'Parent folder ID for connection ID: ' + connectionId + ' not found.' );
            }
          }
          else
          {
            if ( json.errorType && json.errorType == QDesk.enums.ServerErrorType.VALIDATION )
            {
              Ext.each( Ext.ComponentQuery.query( 'form', me.window ), function ( formPanel )
              {
                formPanel.getForm().markInvalid( json.errors );
              } );
            }

            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.SaveSchedulerConnection.updateErrorMsg',
                  connectionName ) );
        }
      }
    } );
  },


  /**
   * Closes the scheduler tab (scheduler view) of the connection that has been updated.
   */
  _closeSchedulerTabForConnection : function ( connectionId )
  {
    var me = this;

    // TODO: maybe we would rather reopen the tab?

    var schedulerTabItemId = "schedulerTabPanel-" + connectionId;
    Ext.each( Ext.ComponentQuery.query( '#' + schedulerTabItemId ), function ( tabPanel )
    {
      tabPanel.close();
    } );
  }
} );
