/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window with a grid populated with messages for the specified creator.
 */
Ext.define( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.main.scheduler.common.message.MessageQueueWindow',
    'QDesk.controller.command.ViewError'
  ],

  /*
   * Required ID of the connection the messages should be obtained from.
   */
  connectionId : undefined,

  /*
   * Required ID of the messages creator.
   */
  creatorId : undefined,

  /*
   * Required parameter with the message creator type ({@link QDesk.enums.message.MessageCreatorType}).
   */
  creatorType : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.creatorId )
      Ext.Error.raise( 'Required config parameter missing: creatorId' );

    if ( !config.creatorType )
      Ext.Error.raise( 'Required config parameter missing: creatorType' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'MessageQueueWindow', {
      qdesk : {
        connectionId : me.connectionId
      }
    } );

    me._bindEventHandlers( window );

    var store = window.down( '#messageQueueGrid' ).getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.messageActionListForCreator;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator.maskLoadMsg' ) );

    // initial load of data
    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        me.unmask();

        if ( success )
        {
          window.show();

          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    window.on( 'afterrender', function ()
    {
      window.maximize();
      window.toFront( false );
    } );

    var grid = window.down( '#messageQueueGrid' );

    var store = grid.getStore();

    // in before load we add all required parameters
    store.on( 'beforeload', function ( store, operation )
    {
      grid.down( '#refreshButton' ).setDisabled( true );
      grid.down( '#maxFetchDateSelect' ).setDisabled( true );
      grid.down( '#maxFetchSizeSelect' ).setDisabled( true );

      var params = operation.getParams() || {};

      var maxFetchDate = grid.down( '#maxFetchDateSelect' ).getValue();
      var maxFetchSize = grid.down( '#maxFetchSizeSelect' ).getValue();

      Ext.apply( params, {
        connectionId : me.connectionId,
        creatorId : me.creatorId,
        creatorType : me.creatorType,
        maxFetchDate : maxFetchDate ? maxFetchDate : null,
        maxFetchSize : maxFetchSize ? maxFetchSize : null
      } );

      operation.setParams( params );
    } );

    store.on( 'load', function ( store, operation )
    {
      grid.down( '#refreshButton' ).setDisabled( false );
      grid.down( '#maxFetchDateSelect' ).setDisabled( false );
      grid.down( '#maxFetchSizeSelect' ).setDisabled( false );
    } );

    // opens a window showing the last error description of the specified message
    grid.down( '#lastErrorDescColumn' ).on( 'requestViewError', function ( lastErrorDesc )
    {
      Ext.create( 'QDesk.controller.command.ViewError', {
        maskEl : window.getEl(),
        errorText : lastErrorDesc
      } ).execute();
    } );

    grid.down( '#refreshButton' ).on( 'click', function ()
    {
      me._refreshGrid( window );
    } );

    grid.down( '#maxFetchDateSelect' ).on( 'change', function ( combo, newValue, oldValue )
    {
      me._refreshGrid( window );
    } );

    grid.down( '#maxFetchSizeSelect' ).on( 'change', function ( combo, newValue, oldValue )
    {
      me._refreshGrid( window );
    } );
  },


  /**
   * Refreshes the contents of the message grid.
   * @param window the messages window.
   * @private
   */
  _refreshGrid : function ( window )
  {
    var me = this;

    var grid = window.down( '#messageQueueGrid' );

    var store = grid.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.messageActionListForCreator;

    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        if ( !success )
        {
          store.removeAll( false );

          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator.errorMsg' ) );
        }
      }
    } );
  }
} );
