/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the executing job's log.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.ViewExecutingJobLog', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob',
    'QDesk.enums.LoggingEventPriority',
    'QDesk.enums.ProductEdition',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.Const',
    'QDesk.view.main.ViewLogWindow'
  ],

  /*
   * Required config property containing the connection ID.
   */
  connectionId : undefined,

  /**
   * Required config property containing the executing job.
   */
  executingJob : undefined,

  /**
   * Internal flag indicating if the view log window has been closed.
   */
  stopLoading : false,

  /**
   * Flag indicating if auto-scroll is enabled.
   */
  autoScrollEnabled : true,

  /**
   * Interval between individual log data fetches in ms.
   */
  fetchInterval : 1000,

  /**
   * Counter of successive log data fetch errors.
   */
  fetchErrorCount : 0,

  /**
   * Maximum number of successive log data fetch errors before we give up.
   */
  maxFetchErrors : 5,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.executingJob )
      Ext.Error.raise( 'Required config parameter missing: executingJob' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    // jobExecutionId is available for Quartz >= 2.1.1
    var jobExecutionId = me.executingJob.get( 'jobExecutionId' );
    if ( Ext.isEmpty( jobExecutionId ) )
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewExecutingJobLog.warnViewLogNotSupported' ) );
    }
    else
    {
      var window = Ext.widget( 'ViewLogWindow' );
      window.on( 'afterrender', function ()
      {
        window.maximize();
        window.toFront( false );
      } );

      window.on( 'close', function ()
      {
        me.stopLoading = true;
      } );

      var editor = window.down( '#editor' );

      /*
       * Set focus on the editor so that the user can immediately use CTRL+A to
       * select all text in the editor. The delay is used to defer setting of the
       * focus to avoid a known issue (see http://forums.ext.net/showthread.php?3315-CLOSED-Set-Focus-to-textfield-in-logon-window).
       */
      editor.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );

      editor.setValue( '<pre id="logText" style="margin-bottom: 0px; margin-top: 0px;"></pre><img id="spinner" src="images/icon/loading-16x16.gif"/>' );

      /*
       * Auto-scroll button logic.
       */
      var autoScrollButton = window.down( 'toolbar #autoScrollButton' );
      autoScrollButton.setDisabled( false );
      autoScrollButton.toggle( me.autoScrollEnabled );

      autoScrollButton.on( 'toggle', function ( button, pressed )
      {
        me.autoScrollEnabled = pressed;

        // scroll to the bottom without waiting for the data
        me._maybeScrollToBottom( editor );
      }, me );

      /*
       * Interrupt job button logic.
       */
      var interruptJobButton = window.down( 'toolbar #interruptJobButton' );
      interruptJobButton.setDisabled( false );
      interruptJobButton.on( 'click', function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob', {
          connectionId : me.connectionId,
          executingJob : me.executingJob
        } ).execute();
      } );

      window.show();

      me._fetchLogData( editor, jobExecutionId, 0 );
    }
  },


  _fetchLogData : function ( editor, jobExecutionId, fromIndex )
  {
    var me = this;

    // if the log window has been closed, stop polling for new log data
    if ( me.stopLoading )
      return;

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execHistoryActionExecutingJobLog,
      params : {
        connectionId : me.connectionId,
        fromIndex : fromIndex,
        jobExecutionId : jobExecutionId
      },
      scope : me,
      callback : function ( opts, success, resp )
      {
        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            // reset the error count with each successful fetch
            me.fetchErrorCount = 0;

            var jobStarted = json.data.jobStarted;
            var jobStillExecuting = json.data.jobStillExecuting;
            var loggingInterceptionEnabled = json.data.loggingInterceptionEnabled;
            var loggingEvents = json.data.loggingEvent;

            if ( jobStarted )
            {
              if ( loggingEvents.length > 0 )
              {
                var nodes = me._createLoggingEventNodes( editor.getDoc(), loggingEvents );
                var logTextDom = editor.getDoc().getElementById( 'logText' );

                Ext.each( nodes, function ( node )
                {
                  logTextDom.appendChild( node );
                } );

                // scroll to the bottom of the editor in case the auto-scroll is enabled
                me._maybeScrollToBottom( editor );

                // enable the save button
                editor.up( 'window' ).down( 'toolbar #saveButton' ).setDisabled( false );
              }

              if ( jobStillExecuting )
              {
                // job still running, keep fetching the logging events
//                QDesk.Log.info( 'Job (job execution ID: ' + jobExecutionId +
//                    ') is still executing. About to fetch new data in ' + me.fetchInterval + ' ms' );

                Ext.defer( me._fetchLogData, me.fetchInterval, me, [editor, jobExecutionId, fromIndex + loggingEvents.length] );
              }
              else
              {
                // job finished executing - remove the spinner
//                QDesk.Log.info( 'Job (job execution ID: ' + jobExecutionId +
//                    ') finished executing. Stopping data fetching.' );

                me._hideSpinner( editor );

                // disable the auto-scroll and interrupt buttons
                editor.up( 'window' ).down( 'toolbar #autoScrollButton' ).setDisabled( true );
                editor.up( 'window' ).down( 'toolbar #interruptJobButton' ).setDisabled( true );

                /*
                 * Warn the user about the logging interception not being enabled. We can only determine if the
                 * logging interception was enabled once the job finished executing and there is no intercepted
                 * log data.
                 */
                if ( !loggingInterceptionEnabled )
                {
                  QDesk.util.UxUtils.showWarningDialog(
                      QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewExecutingJobLog.warnLogInterceptionNotEnabledMsg' ) );
                }
              }
            }
            else
            {
              // job has not started yet - keep trying
//              QDesk.Log.info( 'Job (job execution ID: ' + jobExecutionId +
//                  ') has not started yet. About to fetch new data in ' + me.fetchInterval + ' ms.' );

              Ext.defer( me._fetchLogData, me.fetchInterval, me, [editor, jobExecutionId, 0] );
            }
          }
          else
          {
            me._hideSpinner( editor );

            // application error
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          me.fetchErrorCount++;
          if ( me.fetchErrorCount >= me.maxFetchErrors )
          {
            QDesk.Log.warn( 'Error fetching job execution data (job execution ID: ' + jobExecutionId +
                '). Maximum fetch errors (' + me.maxFetchErrors + ') reached, no further retries will be performed.' );

            me._hideSpinner( editor );

            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewExecutingJobLog.errorFetchDataMsg' ) );
          }
          else
          {
            QDesk.Log.warn( 'Error fetching job execution data (job execution ID: ' + jobExecutionId +
                '). Retrying in ' + me.fetchInterval + ' ms.' );

            Ext.defer( me._fetchLogData, me.fetchInterval, me, [editor, jobExecutionId, fromIndex] );
          }
        }
      }
    } );
  },


  /**
   * Formats the specified list of logging events and returns list of DOM nodes
   * to be added to the HTML editor's log data content area.
   * @param doc a DOM document.
   * @param loggingEvents an array of logging event objects with the following structure:
   * <pre>
   *   {
   *     priority : <log message priority>,
   *     message: <log message>
   *   }
   * </pre>
   * @return the formatted logging events as an HTML string.
   * @private
   */
  _createLoggingEventNodes : function ( doc, loggingEvents )
  {
    var result = [];
    Ext.each( loggingEvents, function ( loggingEvent )
    {
      var priority = loggingEvent.priority;

      var priorityColor;
      if ( priority == QDesk.enums.LoggingEventPriority.TRACE )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.trace' )
      }
      else if ( priority == QDesk.enums.LoggingEventPriority.DEBUG )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.debug' )
      }
      else if ( priority == QDesk.enums.LoggingEventPriority.INFO )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.info' )
      }
      else if ( priority == QDesk.enums.LoggingEventPriority.WARN )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.warn' )
      }
      else if ( priority == QDesk.enums.LoggingEventPriority.ERROR )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.error' )
      }

      var loggingEventEl = doc.createElement( 'span' );
      loggingEventEl.innerHTML = loggingEvent.message + '<br/>';
      loggingEventEl.setAttribute( 'style', 'color: ' + priorityColor );

      result.push( loggingEventEl );
    } );

    return result;
  },


  /**
   * Hides the spinner icon.
   * @param editor the HTML editor.
   * @private
   */
  _hideSpinner : function ( editor )
  {
    var spinnerDom = editor.getDoc().getElementById( 'spinner' );
    spinnerDom.parentNode.removeChild( spinnerDom );
  },


  /**
   * Scrolls the specified HTML editor to the bottom.
   * @param editor the HTML editor.
   * @private
   */
  _maybeScrollToBottom : function ( editor )
  {
    var me = this;

    if ( me.autoScrollEnabled )
    {
      var editorEl = editor.getEl();
      var iframeEl = editorEl.down( 'iframe' );
      var iframe = iframeEl.dom;

      var scrollHeight =
          iframe.contentWindow.document.body.scrollHeight -
          iframe.style.height.replace( "px", "" );

      // Add 20px to make sure the spinner icon is always visible. If those 20pxs are not
      // added, then when the Log window is not maximized, then the spinner can be partially hidden.
      iframe.contentWindow.scrollTo( 0, scrollHeight + 20 );
    }
  }
} );
