/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Job Chains tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.QuartzControllerUtils',

    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid',
    'QDesk.controller.command.scheduler.quartz.chain.AddJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EditJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain',

    'QDesk.enums.message.MessageCreatorType',

    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerJobChainsTab = QDesk.controller.QuartzControllerUtils.getSchedulerJobChainsTab( me.connectionId );

    me._bindEventHandlers( schedulerJobChainsTab );

    schedulerJobChainsTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Job Chains tab to their respective handlers.
   */
  _bindEventHandlers : function ( schedulerJobChainsTab )
  {
    var me = this;
    me._bindEventHandlersJobChainsGrid( schedulerJobChainsTab );
  },


  /**
   * Binds the Job Chains grid events to their handlers.
   */
  _bindEventHandlersJobChainsGrid : function ( schedulerJobChainsTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobChaining &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var jobChainsGrid = schedulerJobChainsTab.down( '#jobChainsGrid' );

      // refresh button
      jobChainsGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshJobChainsGrid( me.connectionId );
          } );

      jobChainsGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onJobChainUnselected( schedulerJobChainsTab );
        }
        else
        {
          me._onJobChainSelected( selectedRecords[ 0 ], schedulerJobChainsTab );
        }
      } );

      var filterQueryText = jobChainsGrid.down( '#filterQueryText' );

      var agentAvailable = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId );
      if ( agentAvailable )
      {
        var jobChainsIndexEnabled = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzJobChainsIndexEnabled( me.connectionId );
        if ( !jobChainsIndexEnabled )
        {
          filterQueryText.emptyText =
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab.jobChainsIndexDisabledEmptyText' );

          filterQueryText.applyEmptyText();
        }
      }
      else
      {
        filterQueryText.emptyText =
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab.indexNoAgentEmptyText' );

        filterQueryText.applyEmptyText();
      }

      jobChainsGrid.getStore().on( 'beforeload', function ( store, operation )
      {
        jobChainsGrid.down( '#refreshButton' ).disable();
        jobChainsGrid.down( '#addJobChainButton' ).disable();
        jobChainsGrid.down( '#deleteJobChainButton' ).disable();
        jobChainsGrid.down( '#editJobChainButton' ).disable();
        jobChainsGrid.down( '#cloneJobChainButton' ).disable();
        filterQueryText.disable();

        /*
         * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
         * Therefore we need to force-fire this event by clearing the selection.
         */
        jobChainsGrid.getSelectionModel().deselectAll();
      } );

      jobChainsGrid.getStore().on( 'load', function ( store, records, successful, operation )
      {
        jobChainsGrid.down( '#refreshButton' ).enable();

        // enable only if the job chains index is enabled in the agent
        if ( jobChainsIndexEnabled )
        {
          filterQueryText.enable();
        }

        if ( successful )
        {
          jobChainsGrid.down( '#addJobChainButton' ).enable();
        }
        else
        {
          store.removeAll();

          var rawData = this.getProxy().getReader().rawData;
          if ( rawData && !rawData.success )  // we better check the success flag to make sure there are actually some errors
          {
            QDesk.util.UxUtils.showJsonResponseErrors( rawData );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.chain.InitializeSchedulerJobChainsTab.errorLoadingJobChainsMsg' ) );
          }
        }
      } );

      jobChainsGrid.down( '#addJobChainButton' ).on( 'click', function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.AddJobChain', {
          connectionId : me.connectionId,
          /*
           * Upon success refresh the job chains tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
              connectionId : me.connectionId
            } ).execute();
          }
        } ).execute();
      } );

      jobChainsGrid.down( '#deleteJobChainButton' ).on( 'click', function ()
      {
        var jobChain = jobChainsGrid.getSelectionModel().getSelection()[ 0 ];
        me._deleteJobChain( jobChain );
      } );

      jobChainsGrid.down( '#editJobChainButton' ).on( 'click', function ()
      {
        var jobChain = jobChainsGrid.getSelectionModel().getSelection()[ 0 ];
        me._editJobChain( jobChain );
      } );

      jobChainsGrid.down( '#cloneJobChainButton' ).on( 'click', function ()
      {
        var jobChain = jobChainsGrid.getSelectionModel().getSelection()[ 0 ];
        me._cloneJobChain( jobChain );
      } );

      /* JobChainGrid context menu actions */
      jobChainsGrid.on( 'enableJobChain', me._enableJobChain, me );
      jobChainsGrid.on( 'disableJobChain', me._disableJobChain, me );
      jobChainsGrid.on( 'deleteJobChain', me._deleteJobChain, me );
      jobChainsGrid.on( 'editJobChain', me._editJobChain, me );
      jobChainsGrid.on( 'cloneJobChain', me._cloneJobChain, me );
    }
  },


  /**
   * Invoked when a job chain record in the SchedulerJobChainsGrid in the SchedulerJobChainsTab
   * is selected.
   * @param jobChain a QDesk.model.scheduler.quartz.QuartzJobChain instance representing the selected
   * job chain.
   * @param schedulerJobChainsTab the scheduler Job Chains tab.
   */
  _onJobChainSelected : function ( jobChain, schedulerJobChainsTab )
  {
    var me = this;

    var jobChainsGrid = schedulerJobChainsTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteJobChainButton' ).enable();
    jobChainsGrid.down( '#editJobChainButton' ).enable();
    jobChainsGrid.down( '#cloneJobChainButton' ).enable();
  },


  /**
   * Invoked when a job chain record in the SchedulerJobChainsGrid in the SchedulerJobChainsTab
   * is unselected.
   * @param schedulerJobChainsTab the scheduler Job Chains tab.
   */
  _onJobChainUnselected : function ( schedulerJobChainsTab )
  {
    var me = this;

    var jobChainsGrid = schedulerJobChainsTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteJobChainButton' ).disable();
    jobChainsGrid.down( '#editJobChainButton' ).disable();
    jobChainsGrid.down( '#cloneJobChainButton' ).disable();
  },


  /**
   * Invoked to refresh the data in the scheduler job chains grid.
   * @param connectionId a connection ID.
   */
  _refreshJobChainsGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
      connectionId : connectionId
    } ).execute();
  },


  _enableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _disableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _deleteJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _editJobChain : function ( jobChains )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain', {
      connectionId : me.connectionId,
      jobChainId : jobChains.get( 'id' ),
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _cloneJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain', {
      connectionId : me.connectionId,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  }
} );
