/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Currently Executing Jobs tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.scheduler.quartz.ViewExecutingJobLog',
    'QDesk.controller.command.scheduler.quartz.ViewJobDataMap',
    'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsGrid',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var executingJobsTab = QDesk.controller.QuartzControllerUtils.getExecutingJobsTab( me.connectionId );

    me._bindEventHandlers( executingJobsTab );

    executingJobsTab.setInitialized( true );
  },


  /**
   * Binds events from the jobs tab to their respective handlers.
   */
  _bindEventHandlers : function ( executingJobsTab )
  {
    var me = this;
    me._bindEventHandlersExecutingJobsGrid( executingJobsTab );

    // no views
  },


  /**
   * Binds the jobs grid events to their handlers.
   */
  _bindEventHandlersExecutingJobsGrid : function ( executingJobsTab )
  {
    var me = this;

    var executingJobsGrid = executingJobsTab.down( '#executingJobsGrid' );

    // refresh button
    executingJobsGrid.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me._refreshExecutingJobsGrid( me.connectionId );
        } );

    var interruptJobButton = executingJobsGrid.down( '#interruptJobButton' );
    interruptJobButton.on( 'click', function ()
    {
      var executingJob = executingJobsGrid.getSelectionModel().getSelection()[0];
      me._interruptJob( executingJob );
    }, me );

    var filterByJobGroupCombo = executingJobsGrid.down( '#filterByJobGroupCombo' );
    filterByJobGroupCombo.on( 'change', function ( combo, newValue )
    {
      executingJobsGrid.getStore().clearFilter( true );

      if ( newValue != null )
      {
        executingJobsGrid.getStore().addFilter( [
          Ext.create( 'Ext.util.Filter', {
            property : 'jobDetail.group.name',
            value : newValue
          } )
        ], false );
      }

      // apply, or clear the filter and load the first page
      executingJobsGrid.getStore().loadPage( 1 );
    } );

    var filterQueryText = executingJobsGrid.down( '#filterQueryText' );

    // event fired by the actioncolumn in the ExecutingJobsGrid
    executingJobsGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
        function ( jobDataMap )
        {
          me._viewJobDataMap( jobDataMap );
        } );

    // event fired by the actioncolumn in the ExecutingJobsGrid
    executingJobsGrid.down( '#logColumn' ).on( 'requestViewLog',
        function ( executingJob )
        {
          me._viewLog( me.connectionId, executingJob );
        } );

    executingJobsGrid.getStore().on( 'beforeload', function ( store, operation )
    {
      executingJobsGrid.down( '#refreshButton' ).disable();
      filterByJobGroupCombo.disable();
      filterQueryText.disable();

      /*
       * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
       * Therefore we need to force-fire this event by clearing the selection.
       */
      executingJobsGrid.getSelectionModel().deselectAll();
    } );

    executingJobsGrid.getStore().on( 'load', function ( executingJobStore, records, successful, operation )
    {
      executingJobsGrid.down( '#refreshButton' ).enable();

      if ( successful )
      {
        filterByJobGroupCombo.enable();
        filterQueryText.enable();
      }
      else
      {
        executingJobStore.removeAll();

        QDesk.util.UxUtils.showErrorDialog(
            QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab.errorLoadingExecutingJobsMsg' ) );
      }
    } );

    executingJobsGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
    {
      if ( selectedRecords.length == 0 )
      {
        me._onJobUnselected( executingJobsTab );
      }
      else
      {
        me._onJobSelected( selectedRecords[0], executingJobsTab );
      }
    } );
  },


  /**
   * Refreshes the currently executing jobs grid.
   * @param connectionId a connection ID.
   * @private
   */
  _refreshExecutingJobsGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsGrid', {
      connectionId : connectionId
    } ).execute();
  },


  /**
   * Invoked when a job in the ExecutingJobsGrid in the ExecutingJobsTab is selected.
   * @param executingJob a QDesk.model.scheduler.quartz.QuartzExecutingJob instance representing the selected job.
   * @param executingJobsTab the executing jobs tab.
   */
  _onJobSelected : function ( executingJob, executingJobsTab )
  {
    var me = this;
    var executingJobsGrid = executingJobsTab.down( '#executingJobsGrid' );
    var interruptJobButton = executingJobsGrid.down( '#interruptJobButton' );
    interruptJobButton.enable();
  },


  /**
   * Invoked when a job in the ExecutingJobsGrid in the ExecutingJobsTab is unselected.
   * @param executingJobsTab an ExecutingJobsTab.
   */
  _onJobUnselected : function ( executingJobsTab )
  {
    var me = this;
    var executingJobsGrid = executingJobsTab.down( '#executingJobsGrid' );
    var interruptJobButton = executingJobsGrid.down( '#interruptJobButton' );
    interruptJobButton.disable();
  },


  /**
   * Invoked to interrupt the specified executing job.
   * @param executingJob the executing job.
   */
  _interruptJob : function ( executingJob )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob', {
      connectionId : me.connectionId,
      executingJob : executingJob
    } ).execute();
  },


  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap a job data map.
   * @param effectiveJobDataMap an effective job data map.
   */
  _viewJobDataMap : function ( jobDataMap, effectiveJobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      jobDataMap : jobDataMap,
      effectiveJobDataMap : effectiveJobDataMap
    } ).execute();
  },


  /**
   * Opens a window showing the log of the executing job.
   * @param connectionId a connection ID.
   * @param executingJob a {@link QDesk.model.scheduler.quartz.QuartzExecutingJob} instance.
   */
  _viewLog : function ( connectionId, executingJob )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewExecutingJobLog', {
      connectionId : connectionId,
      executingJob : executingJob
    } ).execute();
  }
} );
