/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Statistics view in the scheduler view panel's Jobs tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobStatisticsView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.main.scheduler.quartz.stats.ExecCountChart',
    'QDesk.view.main.scheduler.quartz.stats.ExecTimeChart'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected job.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionStatistics &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var jobsTab = QDesk.controller.QuartzControllerUtils.getJobsTab( me.connectionId );
      var jobsGrid = jobsTab.down( '#jobsGrid' );

      var selectedJobDetails = jobsGrid.getSelectionModel().getSelection();

      var statisticsPanel = jobsTab.down( '#statisticsPanel' );

      var statisticsType = statisticsPanel.getStatisticsType();
      var periodLength = statisticsPanel.getPeriodLength();
      var periodStart = statisticsPanel.getPeriodStart();

      /*
       * Remove chart description text and remove the chart component.
       */
      var chartDescriptionPanel = statisticsPanel.down( '#chartDescription' );
      chartDescriptionPanel.setVisible( false );
      chartDescriptionPanel.update( '' );

      statisticsPanel.down( '#chartContainer' ).removeAll();

      // disable all fields in the statistics' panel toolbar
      var toolbar = statisticsPanel.down( 'toolbar' );
      Ext.each( Ext.ComponentQuery.query( 'button, field', toolbar ),
          function ( item )
          {
            item.disable();
          }
      );

      if ( me.populate )
      {
        if ( Ext.isEmpty( selectedJobDetails ) )
        {
          // no job selected - ignore
        }
        else
        {
          var jobDetail = selectedJobDetails[0];

          if ( statisticsType && periodLength )
          {
            if ( periodStart == null )
            {
              me._periodsNotAvailable( statisticsPanel, jobDetail, statisticsType, periodLength );
            }
            else
            {
              me._renderChartAndDescription( statisticsPanel, jobDetail, statisticsType, periodLength, periodStart );
            }
          }
        }
      }
      else
      {
        // don't populate - ignore
      }
    }
  },


  _periodsNotAvailable : function ( statisticsPanel, jobDetail, statisticsType, periodLength )
  {
    var me = this;

    var toolbar = statisticsPanel.down( 'toolbar' );

    // enable the refresh button + all fields in the statistics' panel toolbar but the period selection controls
    Ext.each( Ext.ComponentQuery.query( '#refreshButton, field', toolbar ),
        function ( item )
        {
          if ( !Ext.String.endsWith( item.getItemId(), 'PeriodSelect' ) )
            item.enable();
        }
    );

    statisticsPanel._reloadAvailablePeriods();
  },


  _renderChartAndDescription : function ( statisticsPanel, jobDetail, statisticsType, periodLength, periodStart )
  {
    var me = this;

    var toolbar = statisticsPanel.down( 'toolbar' );

    // enable the refresh button + all fields in the statistics' panel toolbar
    Ext.each( Ext.ComponentQuery.query( '#refreshButton, field', toolbar ),
        function ( item )
        {
          item.enable();
        }
    );

    var chart;

    switch ( statisticsType )
    {
      case QDesk.enums.scheduler.quartz.JobStatisticsType.JOB_EXEC_COUNT:
        chart = Ext.create( 'QDesk.view.main.scheduler.quartz.stats.ExecCountChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );
        break;

      case QDesk.enums.scheduler.quartz.JobStatisticsType.JOB_EXEC_TIME:
        chart = Ext.create( 'QDesk.view.main.scheduler.quartz.stats.ExecTimeChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );
        break;

      default:
        Ext.Error.raise( 'Unexpected job statistics type: ' + statisticsType );
    }

    var store = chart.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.jobStatisticsActionChartData;

    store.getProxy().extraParams = {
      connectionId : me.connectionId,
      statisticsType : statisticsType,
      periodLength : periodLength,
      periodStart : periodStart,
      jobGroupName : jobDetail.get( 'group' ).name,
      jobName : jobDetail.get( 'name' )
    };

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobStatisticsView.maskLoadMsg' ) );

    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        me.unmask();

        if ( success )
        {
          var chartDescriptionPanel = statisticsPanel.down( '#chartDescription' );

          chartDescriptionPanel.update( QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.JobStatisticsType.description.' +
              statisticsType ) );

          chartDescriptionPanel.setVisible( true );

          statisticsPanel.down( '#chartContainer' ).add( chart );

          // enable remaining buttons (eg. save chart) that have been disabled
          Ext.each( Ext.ComponentQuery.query( 'button', toolbar ),
              function ( item )
              {
                item.enable();
              }
          );

          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobStatisticsView.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
