/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various Quartz trigger specific utility methods.
 */
Ext.define( 'QDesk.util.quartz.QuartzTriggerUtils', {
  requires : [
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.enums.scheduler.quartz.TriggerType',

    'QDesk.model.scheduler.quartz.QuartzTrigger',

    'QDesk.util.CommonUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.quartz.QuartzEnumUtils'
  ],

  singleton : true,


  /**
   * Creates a clone of the specified QDesk.model.scheduler.quartz.QuartzTrigger instance.
   * @param trigger a QDesk.model.scheduler.quartz.QuartzTrigger instance.
   * @return the clone.
   */
  cloneTrigger : function ( trigger )
  {
    var clone = trigger.copy();  // does not clone the jobDataMap array

    var jobDataMap = trigger.get( 'jobDataMap' );

    if ( jobDataMap )
    {
      // assert Ext.isArray(jobDataMap)
      clone.set( 'jobDataMap', Ext.Array.clone( jobDataMap ) );
    }

    return clone;
  },


  /**
   * Creates a QDesk.model.scheduler.quartz.QuartzTrigger model instance and populates it with
   * the specified EditXXXTriggerForm field values.
   * @param fieldValues a EditXXXTriggerForm field values.
   * @param jobDataMap an array of QDesk.model.scheduler.quartz.QuartzJobDataMapEntry instances.
   * @return the created QDesk.model.scheduler.quartz.QuartzTrigger model instance.
   */
  createTrigger : function ( fieldValues, jobDataMap )
  {
    var me = this;

    var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger' );

    if ( !Ext.isEmpty( fieldValues.id ) )
      trigger.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.type ) )
      trigger.set( 'type', fieldValues.type );

    if ( !Ext.isEmpty( fieldValues.jobGroupName ) )
      trigger.set( 'jobGroup', { name : fieldValues.jobGroupName } );

    if ( !Ext.isEmpty( fieldValues.jobName ) )
      trigger.set( 'jobName', fieldValues.jobName );

    /* Start of common trigger attributes */
    if ( !Ext.isEmpty( fieldValues.name ) )
      trigger.set( 'name', fieldValues.name );

    if ( !Ext.isEmpty( fieldValues.group ) )
      trigger.set( 'group', { name : fieldValues.group } );

    if ( !Ext.isEmpty( fieldValues.description ) )
      trigger.set( 'description', fieldValues.description );

    // start time in the scheduler's (JVM Agent's) time zone
    if ( !Ext.isEmpty( fieldValues.startTime ) )
      trigger.set( 'startTime', fieldValues.startTime );

    // start time in the scheduler's (JVM Agent's) time zone
    if ( !Ext.isEmpty( fieldValues.endTime ) )
      trigger.set( 'endTime', fieldValues.endTime );

    if ( !Ext.isEmpty( fieldValues.priority ) )
      trigger.set( 'priority', fieldValues.priority );

    if ( !Ext.isEmpty( fieldValues.misfireInstruction ) )
      trigger.set( 'misfireInstruction', fieldValues.misfireInstruction );

    if ( !Ext.isEmpty( fieldValues.calendarName ) )
      trigger.set( 'calendar', { id : fieldValues.calendarName } );

    if ( !Ext.isEmpty( jobDataMap ) )
      trigger.set( 'jobDataMap', jobDataMap );

    /* End of common trigger attributes */

    /* Start of trigger-type specific attributes */
    if ( fieldValues.type == QDesk.enums.scheduler.quartz.TriggerType.SIMPLE )
    {
      trigger.set( 'repeatCount', fieldValues.repeatCountIndefinitely ? -1 : fieldValues.repeatCount );

      if ( !Ext.isEmpty( fieldValues.repeatInterval ) )
        trigger.set( 'repeatInterval', fieldValues.repeatInterval );
    }
    else if ( fieldValues.type == QDesk.enums.scheduler.quartz.TriggerType.CRON )
    {
      if ( !Ext.isEmpty( fieldValues.cronExpression ) )
        trigger.set( 'cronExpression', fieldValues.cronExpression );

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        trigger.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else if ( fieldValues.type == QDesk.enums.scheduler.quartz.TriggerType.CALENDAR_INTERVAL )
    {
      if ( !Ext.isEmpty( fieldValues.repeatIntervalUnit ) )
        trigger.set( 'repeatIntervalUnit', fieldValues.repeatIntervalUnit );

      if ( !Ext.isEmpty( fieldValues.repeatInterval ) )
        trigger.set( 'repeatInterval', fieldValues.repeatInterval );

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        trigger.set( 'timeZone', { id : fieldValues.timeZone } );

      if ( !Ext.isEmpty( fieldValues.preserveHourOfDayAcrossDaylightSavings ) )
        trigger.set( 'preserveHourOfDayAcrossDaylightSavings', fieldValues.preserveHourOfDayAcrossDaylightSavings );

      if ( !Ext.isEmpty( fieldValues.skipDayIfHourDoesNotExist ) )
        trigger.set( 'skipDayIfHourDoesNotExist', fieldValues.skipDayIfHourDoesNotExist );
    }
    else if ( fieldValues.type == QDesk.enums.scheduler.quartz.TriggerType.DAILY_TIME_INTERVAL )
    {
      if ( !Ext.isEmpty( fieldValues.repeatIntervalUnit ) )
        trigger.set( 'repeatIntervalUnit', fieldValues.repeatIntervalUnit );

      if ( !Ext.isEmpty( fieldValues.repeatInterval ) )
        trigger.set( 'repeatInterval', fieldValues.repeatInterval );

      trigger.set( 'repeatCount', fieldValues.repeatCountIndefinitely ? -1 : fieldValues.repeatCount );

      if ( !Ext.isEmpty( fieldValues.startTimeOfDay ) )
        trigger.set( 'startTimeOfDay', QDesk.util.FormatUtils.formatDayTime( fieldValues.startTimeOfDay ) );

      if ( !Ext.isEmpty( fieldValues.endTimeOfDay ) )
        trigger.set( 'endTimeOfDay', QDesk.util.FormatUtils.formatDayTime( fieldValues.endTimeOfDay ) );

      trigger.set( 'daysOfWeek', me._createWeekdaysArray( fieldValues, 'dayOfWeek' ) );
    }
    else if ( fieldValues.type == QDesk.enums.scheduler.quartz.TriggerType.DATE_INTERVAL )
    {
      if ( !Ext.isEmpty( fieldValues.repeatIntervalUnit ) )
        trigger.set( 'repeatIntervalUnit', fieldValues.repeatIntervalUnit );

      if ( !Ext.isEmpty( fieldValues.repeatInterval ) )
        trigger.set( 'repeatInterval', fieldValues.repeatInterval );
    }
    else if ( fieldValues.type == QDesk.enums.scheduler.quartz.TriggerType.NTH_INCLUDED_DAY )
    {
      if ( !Ext.isEmpty( fieldValues.intervalType ) )
        trigger.set( 'intervalType', fieldValues.intervalType );

      if ( !Ext.isEmpty( fieldValues.nthDay ) )
        trigger.set( 'nthDay', fieldValues.nthDay );

      if ( !Ext.isEmpty( fieldValues.nextFireCutoffInterval ) )
        trigger.set( 'nextFireCutoffInterval', fieldValues.nextFireCutoffInterval );

      if ( !Ext.isEmpty( fieldValues.fireAt ) )
        trigger.set( 'fireAt', QDesk.util.FormatUtils.formatDayTime( fieldValues.fireAt ) );

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        trigger.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else
    {
      Ext.Error.raise( 'Unexpected trigger type: ' + fieldValues.type );
    }

    /* End of trigger-type specific attributes */

    return trigger;
  },


  /**
   * Creates and returns an HTML table for the specified trigger object.
   * This method is invoked from the trigger grids to obtain a formatted
   * HTML code to display trigger properties.
   * @param trigger a trigger object (not a Trigger model instance!).
   * @returns {string}
   */
  createTriggerPropertiesTable : function ( trigger )
  {
    switch ( trigger.type )
    {
      case QDesk.enums.scheduler.quartz.TriggerType.SIMPLE:
        return '<table class="triggerProperties">' +
            '<thead>' +
            '<tr><th class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleName' ) +
            '</th><th class="value">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleValue' ) +
            '</th></tr>' +
            '</thead>' +
            '<tbody>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatInterval' ) +
            '</td><td class="value">' +
            trigger.repeatInterval + ' ms' +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatCount' ) +
            '</td><td class="value">' +
            (trigger.repeatCount == -1 ?
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatCountIndefinitely' ) :
                trigger.repeatCount) +
            '</td></tr>' +

            '</tbody>' +
            '</table>';
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.CRON:
        return '<table class="triggerProperties">' +
            '<thead>' +
            '<tr><th class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleName' ) +
            '</th><th class="value">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleValue' ) +
            '</th></tr>' +
            '</thead>' +
            '<tbody>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.cronExpression' ) +
            '</td><td class="value">' +
            trigger.cronExpression +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.timeZone' ) +
            '</td><td class="value">' +
            (trigger.timeZone == null ? '' : trigger.timeZone.displayName) +
            '</td></tr>' +

            '</tbody>' +
            '</table>';
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.CALENDAR_INTERVAL:
        return '<table class="triggerProperties">' +
            '<thead>' +
            '<tr><th class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleName' ) +
            '</th><th class="value">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleValue' ) +
            '</th></tr>' +
            '</thead>' +
            '<tbody>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatIntervalUnit' ) +
            '</td><td class="value">' +
            QDesk.util.quartz.QuartzEnumUtils.getRepeatIntervalUnitName( trigger.repeatIntervalUnit ) +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatInterval' ) +
            '</td><td class="value">' +
            trigger.repeatInterval +
            '</td></tr>' +

          // Quartz >= 2.1.3
            ( Ext.isEmpty( trigger.timeZone ) ? '' : (
                '<tr><td class="property">' +
                    QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.timeZone' ) +
                    '</td><td class="value">' +
                    trigger.timeZone.displayName +
                    '</td></tr>' )) +

          // Quartz >= 2.1.3
            (Ext.isEmpty( trigger.preserveHourOfDayAcrossDaylightSavings ) ? '' : (
                '<tr><td class="property">' +
                    QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.preserveHourOfDayAcrossDaylightSavings' ) +
                    '</td><td class="value">' +
                    trigger.preserveHourOfDayAcrossDaylightSavings +
                    '</td></tr>' )) +

          // Quartz >= 2.1.3
            (Ext.isEmpty( trigger.skipDayIfHourDoesNotExist ) ? '' : (
                '<tr><td class="property">' +
                    QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.skipDayIfHourDoesNotExist' ) +
                    '</td><td class="value">' +
                    trigger.skipDayIfHourDoesNotExist +
                    '</td></tr>' )) +

            '</tbody>' +
            '</table>';
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.DAILY_TIME_INTERVAL:
        return '<table class="triggerProperties">' +
            '<thead>' +
            '<tr><th class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleName' ) +
            '</th><th class="value">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleValue' ) +
            '</th></tr>' +
            '</thead>' +
            '<tbody>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatIntervalUnit' ) +
            '</td><td class="value">' +
            QDesk.util.quartz.QuartzEnumUtils.getRepeatIntervalUnitName( trigger.repeatIntervalUnit ) +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatInterval' ) +
            '</td><td class="value">' +
            trigger.repeatInterval +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatCount' ) +
            '</td><td class="value">' +
            (trigger.repeatCount == -1 ?
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatCountIndefinitely' ) :
                trigger.repeatCount) +
            '</td></tr>' +

          // trigger.startTimeOfDay is a Date when using the Trigger ExtJS data model,
          // or String when using the TriggerWithJobDetail data model
            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.startTimeOfDay' ) +
            '</td><td class="value">' +
            ( Ext.isDate( trigger.startTimeOfDay ) ?
                QDesk.util.FormatUtils.formatTimeMedium( trigger.startTimeOfDay ) :
                QDesk.util.FormatUtils.formatTimeMedium( QDesk.util.FormatUtils.parseDayTime( trigger.startTimeOfDay ) )) +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.endTimeOfDay' ) +
            '</td><td class="value">' +
          // trigger.endTimeOfDay is a Date when using the Trigger ExtJS data model,
          // or String when using the TriggerWithJobDetail data model
            ( Ext.isDate( trigger.endTimeOfDay ) ?
                QDesk.util.FormatUtils.formatTimeMedium( trigger.endTimeOfDay ) :
                QDesk.util.FormatUtils.formatTimeMedium( QDesk.util.FormatUtils.parseDayTime( trigger.endTimeOfDay ) )) +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.daysOfWeek' ) +
            '</td><td class="value">' +
            QDesk.util.quartz.QuartzTriggerUtils._formatWeekDays( trigger.daysOfWeek ) +
            '</td></tr>' +

            '</tbody>' +
            '</table>';
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.DATE_INTERVAL:
        return '<table class="triggerProperties">' +
            '<thead>' +
            '<tr><th class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleName' ) +
            '</th><th class="value">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleValue' ) +
            '</th></tr>' +
            '</thead>' +
            '<tbody>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatIntervalUnit' ) +
            '</td><td class="value">' +
            QDesk.util.quartz.QuartzEnumUtils.getRepeatIntervalUnitName( trigger.repeatIntervalUnit ) +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.repeatInterval' ) +
            '</td><td class="value">' +
            trigger.repeatInterval +
            '</td></tr>' +

            '</tbody>' +
            '</table>';
        break;

      case QDesk.enums.scheduler.quartz.TriggerType.NTH_INCLUDED_DAY:
        return '<table class="triggerProperties">' +
            '<thead>' +
            '<tr><th class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleName' ) +
            '</th><th class="value">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.titleValue' ) +
            '</th></tr>' +
            '</thead>' +
            '<tbody>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.intervalType' ) +
            '</td><td class="value">' +
            QDesk.util.quartz.QuartzEnumUtils.getIntervalTypeName( trigger.intervalType ) +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.nthDay' ) +
            '</td><td class="value">' +
            trigger.nthDay +
            '</td></tr>' +

          // trigger.fireAt is a Date when using the Trigger ExtJS data model,
          // or String when using the TriggerWithJobDetail data model
            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.fireAt' ) +
            '</td><td class="value">' +
            ( Ext.isDate( trigger.fireAt ) ?
                QDesk.util.FormatUtils.formatTimeMedium( trigger.fireAt ) :
                QDesk.util.FormatUtils.formatTimeMedium( QDesk.util.FormatUtils.parseDayTime( trigger.fireAt ) )) +
            '</td></tr>' +

            '<tr><td class="property">' +
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.triggerProperty.timeZone' ) +
            '</td><td class="value">' +
            (trigger.timeZone == null ? '' : trigger.timeZone.displayName) +
            '</td></tr>' +

            '</tbody>' +
            '</table>';
        break;

      default:
        break;
    }
  },


  /**
   * Creates an array of AddTriggerButton's menu items for the specified Quartz scheduler version.
   * @param schedulerVersion the Quartz scheduler version.
   * @returns {Array} the array of menu items.
   */
  createAddTriggerButtonMenuItems : function ( schedulerVersion )
  {
    var addTriggerMenuItems = [
      {
        itemId : 'addSimpleTrigger',
        text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( QDesk.enums.scheduler.quartz.TriggerType.SIMPLE ),
//            + ' '
//            + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
        qdesk : {
          triggerType : QDesk.enums.scheduler.quartz.TriggerType.SIMPLE
        }
      },
      {
        itemId : 'addCronTrigger',
        text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( QDesk.enums.scheduler.quartz.TriggerType.CRON ),
//            + ' '
//            + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
        qdesk : {
          triggerType : QDesk.enums.scheduler.quartz.TriggerType.CRON
        }
      }
    ];

    // schedulerVersion < 2.0.0
    if ( QDesk.util.CommonUtils.versionCompare( schedulerVersion, {major : 2, minor : 0, maintenance : 0} ) == -1 )
    {
      addTriggerMenuItems.push(
          // <= Quartz 1.8.6
          {
            itemId : 'addDateIntervalTrigger',
            text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( QDesk.enums.scheduler.quartz.TriggerType.DATE_INTERVAL ),
//                + ' '
//                + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
            qdesk : {
              triggerType : QDesk.enums.scheduler.quartz.TriggerType.DATE_INTERVAL
            }
          }
      );

      addTriggerMenuItems.push(
          // <= Quartz 1.8.6
          {
            itemId : 'addNthIncludedDayTrigger',
            text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( QDesk.enums.scheduler.quartz.TriggerType.NTH_INCLUDED_DAY ),
//                + ' '
//                + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
            qdesk : {
              triggerType : QDesk.enums.scheduler.quartz.TriggerType.NTH_INCLUDED_DAY
            }
          }
      );
    }
    // schedulerVersion >= 2.0.0
    else if ( QDesk.util.CommonUtils.versionCompare( schedulerVersion, {major : 2, minor : 0, maintenance : 0} ) >= 0 )
    {
      addTriggerMenuItems.push(
          // >= Quartz 2.0.0
          {
            itemId : 'addCalendarIntervalTrigger',
            text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( QDesk.enums.scheduler.quartz.TriggerType.CALENDAR_INTERVAL ),
//                + ' '
//                + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
            qdesk : {
              triggerType : QDesk.enums.scheduler.quartz.TriggerType.CALENDAR_INTERVAL
            }
          }
      );

      // schedulerVersion >= 2.1.0
      if ( QDesk.util.CommonUtils.versionCompare( schedulerVersion, {major : 2, minor : 1, maintenance : 0} ) >= 0 )
      {
        addTriggerMenuItems.push(
            // >= Quartz 2.1.0
            {
              itemId : 'addDailyTimeIntervalTrigger',
              text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( QDesk.enums.scheduler.quartz.TriggerType.DAILY_TIME_INTERVAL ),
//                  + ' '
//                  + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
              qdesk : {
                triggerType : QDesk.enums.scheduler.quartz.TriggerType.DAILY_TIME_INTERVAL
              }
            }
        );
      }
    }

    return addTriggerMenuItems;
  },


  _formatWeekDays : function ( weekdays )
  {
    if ( Ext.isEmpty( weekdays ) )
    {
      return '';
    }
    else
    {
      var result = '';
      Ext.each( weekdays, function ( weekday, index )
      {
        result += QDesk.util.EnumUtils.getWeekdayName( weekday );
        if ( index < weekdays.length - 1 )
          result += '<br/>';
      } );
      return result;
    }
  },


  /**
   * Returns the array of QDesk.enums.Weekday instances for the specified form field values.
   * @param fieldValues form field values.
   * @param fieldPrefix the form field name prefix used to denote weekdays.
   * @return the array of QDesk.enums.Weekday instances.
   */
  _createWeekdaysArray : function ( fieldValues, fieldPrefix )
  {
    var enabledOnWeekdays = [];

    if ( fieldValues[fieldPrefix + 'Monday'] )
      enabledOnWeekdays.push( QDesk.enums.Weekday.MONDAY );

    if ( fieldValues[fieldPrefix + 'Tuesday'] )
      enabledOnWeekdays.push( QDesk.enums.Weekday.TUESDAY );

    if ( fieldValues[fieldPrefix + 'Wednesday'] )
      enabledOnWeekdays.push( QDesk.enums.Weekday.WEDNESDAY );

    if ( fieldValues[fieldPrefix + 'Thursday'] )
      enabledOnWeekdays.push( QDesk.enums.Weekday.THURSDAY );

    if ( fieldValues[fieldPrefix + 'Friday'] )
      enabledOnWeekdays.push( QDesk.enums.Weekday.FRIDAY );

    if ( fieldValues[fieldPrefix + 'Saturday'] )
      enabledOnWeekdays.push( QDesk.enums.Weekday.SATURDAY );

    if ( fieldValues[fieldPrefix + 'Sunday'] )
      enabledOnWeekdays.push( QDesk.enums.Weekday.SUNDAY );

    return enabledOnWeekdays;
  }
} );
