/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.JobDataMapGrid', {
  extend : 'Ext.grid.Panel',

  requires : [
    'QDesk.model.scheduler.quartz.QuartzJobDataMapEntry',

    'Ext.grid.plugin.CellEditing'
  ],

  alias : 'widget.QuartzJobDataMapGrid',

  cls : 'quartzJobDataMapGrid',

  /* QDesk specific data */
  qdesk : {
    // flag indicating whether the grid is editable, or not
    editable : false,

    // job/trigger job data map
    jobDataMap : undefined,

    // effective trigger job data map (null for jobs)
    effectiveJobDataMap : undefined
  },


  initComponent : function ()
  {
    var me = this;

    Ext.apply( this, {
      columns : [
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.JobDataMapGrid.textNameColumn' ),
          flex : 1,
          hideable : false,
          sortable : true,
          dataIndex : 'name',
          field : {
            type : 'textfield'
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.JobDataMapGrid.textValueColumn' ),
          flex : 1,
          hideable : false,
          sortable : false,
          dataIndex : 'value',
          field : {
            type : 'textfield'
          }
        }
      ]
    } );

    me.callParent();
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.jobDataMap )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jobDataMap' );

    if ( config.qdesk.editable == null )
      Ext.Error.raise( 'Required config parameter missing: qdesk.editable' );

    var editable = config.qdesk.editable;

    var data = config.qdesk.jobDataMap;

    var store = Ext.create( 'Ext.data.Store', {
      model : 'QDesk.model.scheduler.quartz.QuartzJobDataMapEntry',
      data : data,
      sorters : [
        {
          property : 'name',
          direction : 'ASC'
        }
      ]
    } );

    Ext.apply( config, {

      // multiselect is enabled only in editing mode
      multiSelect : editable,

      store : store
    } );

    if ( editable )
    {
      var editing = Ext.create( 'Ext.grid.plugin.CellEditing' );
      Ext.apply( config, {

        editing : editing,

        plugins : [
          // plugin that allows for table data editing
          editing
        ],

        // add and remove buttons in the grid
        dockedItems : [
          {
            xtype : 'toolbar',
            items : [
              {
                itemId : 'addJobDataMapEntryButton',
                iconCls : 'addJobDataMapEntry',
                tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.JobDataMapGrid.tooltipAddJobDataMapEntryButton' ),
                handler : me._addNewEntry,
                scope : me,
                disabled : false
              },
              {
                itemId : 'deleteJobDataMapEntryButton',
                iconCls : 'deleteJobDataMapEntry',
                tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.JobDataMapGrid.tooltipDeleteJobDataMapEntryButton' ),
                handler : me._deleteSelectedEntries,
                scope : me,
                disabled : true
              }
            ]
          }
        ],

        listeners : {
          itemcontextmenu : function ( grid, record, item, index, event )
          {
            var me = this;
            event.stopEvent();
            var rowCtxMenu = me._createRowCtxMenu( record );
            rowCtxMenu.showAt( event.getXY() );
          }
        }
      } );
    }

    me.callParent( [config] );

    if ( editable )
      me.getSelectionModel().on( 'selectionchange', me.onSelectChange, me );
  },


  /**
   * Invoked when the grid selection changes. This method enables/disables
   * the Delete button in case no record is selected.
   * @param selModel a selection model.
   * @param selections list of selected rows.
   */
  onSelectChange : function ( selModel, selections )
  {
    this.down( '#deleteJobDataMapEntryButton' ).setDisabled( selections.length === 0 );
  },


  /**
   * Invoked when the Add button is clicked. This method
   * adds a new row at the end of the grid.
   */
  _addNewEntry : function ()
  {
    var me = this;

    edit = me.editing;
    edit.cancelEdit();

    var editRowAt = me._getIndexOfRowWithEmptyName();
    if ( editRowAt == null )
    {
      var rec = new QDesk.model.scheduler.quartz.QuartzJobDataMapEntry( {
        name : '',
        value : ''
      } );

      // new row will be added at the end of the grid
      editRowAt = me.store.getCount();

      me.store.insert( editRowAt, rec );

      // need to obtain the current empty row index after the insertion
      editRowAt = me._getIndexOfRowWithEmptyName();
    }

    edit.startEditByPosition( {
      row : editRowAt,
      column : 0
    } );
  },


  /**
   * Invoked when the Delete button is clicked. This method
   * removes the selected job data row from the grid.
   */
  _deleteSelectedEntries : function ()
  {
    var me = this;

    var selections = me.getView().getSelectionModel().getSelection();
    Ext.each( selections, function ( selection )
    {
      me.store.remove( selection );
    } );
  },


  /**
   * Returns true, if there is an empty row in the grid, false otherwise.
   * @return true, if there is an empty row in the grid, false otherwise.
   */
  _getIndexOfRowWithEmptyName : function ()
  {
    var me = this;
    var emptyNameAt = null;

    me.store.data.each( function ( entry, index, length )
    {
      if ( Ext.isEmpty( entry.get( 'name' ) ) )
      {
        emptyNameAt = index;
        return false;
      }
    }, me );

    return emptyNameAt;
  },


  /**
   * Invoked to create a context menu for the specified job data map entry in the JobDataMapGrid row.
   * @param jobDataMapEntry a job data map entry to create the context menu for.
   * @private
   */
  _createRowCtxMenu : function ( jobDataMapEntry )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // delete entry
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.JobDataMapGrid.ctxMenuDelete' ),
          iconCls : 'deleteJobDataMapEntry',
          handler : function ()
          {
            me.store.remove( jobDataMapEntry );
          }
        }
      ]
    } );
  }
} )
;