/*
 * Copyright (c) 2013-2015 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.widget.ConfigurableGrid', {
  extend : 'Ext.grid.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.SaveGridConfig'
  ],

  scrollable : true,

  qdesk : {
    /**
     * Required config property with the grid identification.
     */
    gridId : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    me.callParent( [config] );
  },


  initComponent : function ()
  {
    var me = this;

    me._applyColumnsVisibility( me.qdesk.gridId, me.columns );
    me._applyColumnsOrder( me.qdesk.gridId, me.columns );

    me.callParent();

    me.down( '#saveGridConfigButton' ).on( 'click', function ()
    {
      me._saveChart();
    }, me );
  },


  /**
   * This function is to be overridden by subclasses who wish to provide
   * additional configuration options that should be saved with other
   * grid configuration values.
   */
  getExtraConfig : function ()
  {
  },


  /**
   * Applies the column order configuration to the specified array of columns.
   * @param gridId a grid ID.
   * @param columns an array of columns.
   * @private
   */
  _applyColumnsOrder : function ( gridId, columns )
  {
    var me = this;

    return Ext.Array.sort( columns, function ( col1, col2 )
    {
      var col1ItemId = col1.itemId;
      var col2ItemId = col2.itemId;

      /*
       * Skip columns whose itemId starts with 'ext-' because these are columns
       * without assigned itemId and whose itemId has been generated by ExtJS.
       */
      if ( Ext.String.startsWith( col1ItemId, 'ext-' ) && Ext.String.startsWith( col2ItemId, 'ext-' ) )
      {
        // leave the order as is
        return 0;
      }

      if ( Ext.String.startsWith( col1ItemId, 'ext-' ) )
      {
        // columns without explicit itemId are shown at the beginning (for example the rowexpander column)
        return -1;
      }

      if ( Ext.String.startsWith( col2ItemId, 'ext-' ) )
      {
        // columns without explicit itemId are shown at the beginning (for example the rowexpander column)
        return 1;
      }

      var i = col1ItemId.lastIndexOf( 'Column' );
      if ( i == -1 )
        Ext.Error.raise( 'Column itemId must have \'Column\' suffix. Found itemId: ' + col1ItemId );
      var col1Name = col1ItemId.substr( 0, i );

      i = col2ItemId.lastIndexOf( 'Column' );
      if ( i == -1 )
        Ext.Error.raise( 'Column itemId must have \'Column\' suffix. Found itemId: ' + col2ItemId );
      var col2Name = col2ItemId.substr( 0, i );

      var col1Order = QDesk.ApplicationConfig.get( gridId + '.columnOrder.' + col1Name );
      if ( col1Order == null ) // fallback if column order is not available in the configuration (show column at the end of the grid)
        col1Order = 999;

      var col2Order = QDesk.ApplicationConfig.get( gridId + '.columnOrder.' + col2Name );
      if ( col2Order == null ) // fallback if column order is not available in the configuration (show column at the end of the grid)
        col2Order = 999;

      if ( col1Order == col2Order )
      {
        return 0;
      }
      else if ( col1Order < col2Order )
      {
        return -1;
      }
      else
      {
        return 1;
      }
    } );
  },


  /**
   * Applies the column visibility configuration to the specified array of columns.
   * @param gridId a grid ID.
   * @param columns an array of columns.
   * @private
   */
  _applyColumnsVisibility : function ( gridId, columns )
  {
    var me = this;

    Ext.each( columns, function ( column )
    {
      var colItemId = column.itemId;

      var i = colItemId.lastIndexOf( 'Column' );
      if ( i == -1 )
        Ext.Error.raise( 'Column itemId must have \'Column\' suffix. Found itemId: ' + colItemId );
      var colName = colItemId.substr( 0, i );

      var columnVisible = QDesk.ApplicationConfig.get( gridId + '.columnVisibility.' + colName );
      if ( columnVisible == null )  // fallback - column visible
        columnVisible = true;

      //column.setHidden( !columnVisible );
      column.hidden = !columnVisible;
    } );
  },


  /**
   * Saves the current grid configuration.
   * @private
   */
  _saveChart : function ()
  {
    var me = this;

    var configChanges = {};
    var columnVisibility = [];
    var columnOrder = [];

    Ext.each( me.columns, function ( column, index )
    {
      var columnItemId = column.getItemId();

      /*
       * Skip columns whose itemId starts with 'ext-' because these are columns
       * without assigned itemId and whose itemId has been generated by ExtJS.
       */
      if ( !Ext.String.startsWith( columnItemId, 'ext-' ) )
      {
        var i = columnItemId.lastIndexOf( 'Column' );
        var columnName = columnItemId.substr( 0, i );

        /* Column visibility*/
        var columnVisible = !column.isHidden();
        configChanges[me.qdesk.gridId + '.columnVisibility.' + columnName] = columnVisible;

        columnVisibility.push( {
          column : columnName,
          visible : columnVisible
        } );

        /* Column order */
        configChanges[me.qdesk.gridId + '.columnOrder.' + columnName] = column.getIndex();

        columnOrder.push( {
          column : columnName,
          order : index
        } );
      }
    } );

    var extraConfig = me.getExtraConfig();
    if ( extraConfig )
    {
      Ext.apply( configChanges, extraConfig );
    }

    Ext.create( 'QDesk.controller.command.SaveGridConfig', {
      gridConfig : {
        configChanges : configChanges,
        columnVisibility : columnVisibility,
        columnOrder : columnOrder,
        success : function ()
        {
          me.fireEvent( 'configChanged', {
            gridConfig : {
              configChanges : configChanges,
              columnVisibility : columnVisibility,
              columnOrder : columnOrder
            }
          } );
        }
      }
    } ).execute();
  }
} );
